#!/bin/bash

# set sane defaults
IFS=$' \t\n'
export LANG=C
export LC_NUMERIC=C
export LC_TIME=C

die()
{
  [ "$1" == '0' ] && printf '%s\n' "$2" || printf '%s\n' "$2" 1>&2
  exit "$1"
}

syntax()
{
  die "$1" \
"Usage: op5 sop collect-logs [OPTION].. [-- [NODE]..]

 -h  This syntax help text.
 -1  Read log data starting from this date/time. Any string compatible with
     date(1)'s --date option is accepted. Defaults to 1 month ago.
 -2  Read log data up until this date/time. Like -1. Defaults to now.
 -d  Use this working directory instead of a temporary generated one.
 -t  Store the final tarball at this path.
"
}

msg()
{
  printf '[%s] %s\n' "$(date '+%H:%M:%S')" "$1" 1>&2
}


compress()
{
  (
    msg "Creating tarball: $tarball"

    base="${tarball%.*}"
    [ "${base##*.}" == 'tar' ] && base="${base%.*}"
    base="${base##*/}"

    cd "$wdir" || exit $?
    tar --xform "s|^|$base/|" -cvvzf "$tarball" 'main.log' 'nagios'

  ) || exit $?
}

nodetest()
{
  local name
  msg 'Verifying node command execution capabilities...'
  for name in $names; do
    mon node rawctrl "$name" --test || exit $?
  done
}

nodepad()
{
  local name
  padlen=0
  for name in $names; do
    [ "${#name}" -ge "$padlen" ] && padlen="${#name}"
  done
}

nodeget() {
  (
  # Enable pipefail, hence the subshell.
  set -o pipefail

  merge="mon log merge -1 '@$range1' -2 '@$range2'"
  range="mon log range '@$range1' '@$range2'"
  strip="mon log filter -v | mon log filter -d"

  s() { SSH_OPTS_EXTRA='-C' mon node rawctrl "$1" "$2"; }

  {
  for name in $names; do
    {
    msg "($name) nagios.log ..."
    s "$name" "$merge nagios | $range | $strip | mon log sort" \
      > "$wdir/nagios/$name.log"

    msg "($name) nagios.log (start/stop entries) ..."
    mon log filter '@/nagios/(event-broker-module|startstop)$' < "$wdir/nagios/$name.log" | \
      mon log inject 'nagios.log' '11'

    msg "($name) merlin/daemon.log ..."
    s "$name" "$merge merlin-daemon | $range | $strip" | \
      mon log inject 'daemon.log' '11'

    msg "($name) merlin/neb.log ..."
    s "$name" "$merge merlin-neb | $range | $strip" | \
      mon log inject 'neb.log' '11'

    } | mon log inject "$name" "$padlen"

  done
  msg 'Finished collecting, now finalizing merged log...'

  } | mon log sort | mon log e2d > "$wdir/main.log" || exit $?

  ) || exit $?
}

rmwd()
{
  [ "${wdir:0:4}" == '/tmp' ] || exit 0

  rm -r --one-file-system -- "$wdir/"{main.log,nagios}
  rmdir -- "$wdir" &> /dev/null
}


range1="$(date -d '1 month ago')"
range2="$(date)"
tarball=''
wdir=''

while [ "$#" -gt '0' ]; do
  case "$1" in
    -h|--help)
      syntax 0
      ;;
    -1)
      [ -n "${2+set}" ] || syntax 1
      range1="$(date -d "$2")" || exit $?
      shift 2
      ;;
    -2)
      [ -n "${2+set}" ] || syntax 1
      range2="$(date -d "$2")" || exit $?
      shift 2
      ;;
    -d)
      [ -n "${2+set}" ] || syntax 1
      wdir="$(readlink -m -- "$2")"
      shift 2
      ;;
    -t)
      [ -n "${2+set}" ] || syntax 1
      tarball="$(readlink -m -- "$2")"
      shift 2
      ;;
    --)
      shift
      break
      ;;
    *)
      syntax 1
      ;;
  esac
done

range1="$(date -d "$range1" '+%s')"
range2="$(date -d "$range2" '+%s')"

if [ -n "$wdir" ]; then
  mkdir -p -- "$wdir" || \
    die '1' "ERROR: Failed creating directory '$wdir'."
else
  wdir="$(mktemp --tmpdir -d "op5logcollect.$(date +%s).$$.XXXXXX")" || \
    exit $?
fi
msg "Created working directory: $wdir"

[ -n "$tarball" ] || \
  tarball="$wdir/op5-monitor-logs_$(date +%Y-%m-%d_%H.%M.%S).tgz"

: > "$tarball" || \
  die '1' "ERROR: Unable to write to tarball file '$tarball'."

trap rmwd EXIT

mkdir -- "$wdir/nagios" || \
 die '1' "ERROR: Failed creating directory '$wdir/nagios'."


if [ -n "$1" ]; then
  names="$HOSTNAME $*"
else
  names="$HOSTNAME $(mon node list)"
fi

printf '\n'
nodetest
printf '\n'
nodepad
nodeget
printf '\n'
compress
printf '\n'

msg 'All done!'
exit 0
